==> mkm-0.12.0.tar.gz: mkm-0.12.0/mkm/crypto/private.py <==
# -*- coding: utf-8 -*-
# ==============================================================================
# MIT License
#
# Copyright (c) 2019 Albert Moky
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# ==============================================================================

from abc import ABC, abstractmethod
from typing import Optional, Any, Dict

from ..types import Wrapper
from .factories import Factories
from .cryptography import key_algorithm
from .asymmetric import SignKey
from .public import PublicKey


class PrivateKey(SignKey, ABC):
    """This class is used to decrypt symmetric key or sign message data

        Asymmetric Cryptography Private Key
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        key data format: {
            algorithm : "RSA", // "ECC", ...
            data      : "{BASE64_ENCODE}",
            ...
        }
    """

    def __eq__(self, other) -> bool:
        if self is other:
            return True
        if isinstance(other, SignKey):
            verify_key = self.public_key
            if verify_key is not None:
                return verify_key.match(key=other)

    @property
    def public_key(self) -> Optional[PublicKey]:
        """
        Get public key from private key

        :return: public key paired to this private key
        """
        raise NotImplemented

    #
    #  Factory methods
    #

    @classmethod
    def generate(cls, algorithm: str):  # -> Optional[PrivateKey]:
        factory = cls.factory(algorithm=algorithm)
        assert isinstance(factory, PrivateKeyFactory), 'key algorithm not support: %s' % algorithm
        return factory.generate_private_key()

    @classmethod
    def parse(cls, key: Any):  # -> Optional[PrivateKey]:
        if key is None:
            return None
        elif isinstance(key, PrivateKey):
            return key
        info = Wrapper.get_dictionary(key)
        # assert info is not None, 'key error: %s' % key
        algorithm = key_algorithm(key=info)
        factory = cls.factory(algorithm=algorithm)
        if factory is None:
            factory = cls.factory(algorithm='*')  # unknown
        # assert isinstance(factory, PrivateKeyFactory), 'key algorithm not support: %s' % algorithm
        return factory.parse_private_key(key=info)

    @classmethod
    def factory(cls, algorithm: str):  # -> Optional[PrivateKeyFactory]:
        return Factories.private_key_factories.get(algorithm)

    @classmethod
    def register(cls, algorithm: str, factory):
        Factories.private_key_factories[algorithm] = factory


class PrivateKeyFactory(ABC):

    @abstractmethod
    def generate_private_key(self) -> Optional[PrivateKey]:
        """
        Generate key

        :return: PrivateKey
        """
        raise NotImplemented

    @abstractmethod
    def parse_private_key(self, key: Dict[str, Any]) -> Optional[PrivateKey]:
        """
        Parse map object to key

        :param key: key info
        :return: PrivateKey
        """
        raise NotImplemented

==> mkm-0.12.6.tar.gz: mkm-0.12.6/mkm/crypto/private.py <==
# -*- coding: utf-8 -*-
# ==============================================================================
# MIT License
#
# Copyright (c) 2019 Albert Moky
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# ==============================================================================

from abc import ABC, abstractmethod
from typing import Optional, Any, Dict

from .asymmetric import SignKey
from .public import PublicKey


class PrivateKey(SignKey, ABC):
    """This class is used to decrypt symmetric key or sign message data

        Asymmetric Cryptography Private Key
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        key data format: {
            algorithm : "RSA", // "ECC", ...
            data      : "{BASE64_ENCODE}",
            ...
        }
    """

    @property
    @abstractmethod
    def public_key(self) -> Optional[PublicKey]:
        """
        Get public key from private key

        :return: public key paired to this private key
        """
        raise NotImplemented

    #
    #  Factory methods
    #

    @classmethod
    def generate(cls, algorithm: str):  # -> Optional[PrivateKey]:
        gf = general_factory()
        return gf.generate_private_key(algorithm=algorithm)

    @classmethod
    def parse(cls, key: Any):  # -> Optional[PrivateKey]:
        gf = general_factory()
        return gf.parse_private_key(key=key)

    @classmethod
    def factory(cls, algorithm: str):  # -> Optional[PrivateKeyFactory]:
        gf = general_factory()
        return gf.get_private_key_factory(algorithm=algorithm)

    @classmethod
    def register(cls, algorithm: str, factory):
        gf = general_factory()
        gf.set_private_key_factory(algorithm=algorithm, factory=factory)


def general_factory():
    from .factory import FactoryManager
    return FactoryManager.general_factory


class PrivateKeyFactory(ABC):

    @abstractmethod
    def generate_private_key(self) -> Optional[PrivateKey]:
        """
        Generate key

        :return: PrivateKey
        """
        raise NotImplemented

    @abstractmethod
    def parse_private_key(self, key: Dict[str, Any]) -> Optional[PrivateKey]:
        """
        Parse map object to key

        :param key: key info
        :return: PrivateKey
        """
        raise NotImplemented
